import * as i1 from '@angular/common';
import { DOCUMENT, CommonModule } from '@angular/common';
import * as i0 from '@angular/core';
import { InjectionToken, ViewContainerRef, Component, Inject, ViewChild, createNgModule, Injector, Injectable, NgModule, Optional, SkipSelf } from '@angular/core';
import { Subject, firstValueFrom } from 'rxjs';

/**
 * Injection token for receiving the dialog container styles in the @type {LazyDialogContainerComponent}.
 *
 * @type {InjectionToken<{
  [klass: string]: any;
}>}
 */
const LAZY_DIALOG_CONTAINER_STYLES = new InjectionToken('LAZY_DIALOG_CONTAINER_STYLES');

/**
 * The lazy dialog container (aka dialog background overlay).
 *
 * @export
 * @class LazyDialogContainerComponent
 * @typedef {LazyDialogContainerComponent}
 */
class LazyDialogContainerComponent {
    /**
     * Creates an instance of LazyDialogContainerComponent.
     *
     * @constructor
     * @public
     * @param {Readonly<{
          [klass: string]: any;
        }>} The dialog container's CSS styles.
     */
    constructor(dialogContainerStyles) {
        this.dialogContainerStyles = dialogContainerStyles;
    }
}
LazyDialogContainerComponent.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.1.2", ngImport: i0, type: LazyDialogContainerComponent, deps: [{ token: LAZY_DIALOG_CONTAINER_STYLES }], target: i0.ɵɵFactoryTarget.Component });
LazyDialogContainerComponent.ɵcmp = i0.ɵɵngDeclareComponent({ minVersion: "14.0.0", version: "14.1.2", type: LazyDialogContainerComponent, selector: "ngx-toolset-lazy-dialog-container", viewQueries: [{ propertyName: "dialogContainer", first: true, predicate: ["dialogContainer"], descendants: true, read: ViewContainerRef }], ngImport: i0, template: "<div class=\"dialog-container\" [ngStyle]=\"dialogContainerStyles\">\n  <ng-template #dialogContainer></ng-template>\n</div>\n", dependencies: [{ kind: "directive", type: i1.NgStyle, selector: "[ngStyle]", inputs: ["ngStyle"] }] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.1.2", ngImport: i0, type: LazyDialogContainerComponent, decorators: [{
            type: Component,
            args: [{ selector: 'ngx-toolset-lazy-dialog-container', template: "<div class=\"dialog-container\" [ngStyle]=\"dialogContainerStyles\">\n  <ng-template #dialogContainer></ng-template>\n</div>\n" }]
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Inject,
                    args: [LAZY_DIALOG_CONTAINER_STYLES]
                }] }]; }, propDecorators: { dialogContainer: [{
                type: ViewChild,
                args: ['dialogContainer', { read: ViewContainerRef }]
            }] } });

var lazyDialogContainer_component = /*#__PURE__*/Object.freeze({
  __proto__: null,
  LazyDialogContainerComponent: LazyDialogContainerComponent
});

/**
 * The reference of a lazy dialog.
 *
 * @export
 * @class LazyDialogRef
 * @typedef {LazyDialogRef}
 * @template DataType
 */
class LazyDialogRef {
    /**
     * Creates an instance of LazyDialogRef.
     *
     * @constructor
     * @public
     * @param {ComponentRef<LazyDialogContainerComponent>} containerComponentRef The reference of the dialog component's container component.
     * @param {?NgModuleRef<unknown>} [moduleRef] The module reference of the dialog component (optional: standalone components aren't related to a NgModule).
     * @param {?DataType} [data] The data of the lazy dialog.
     */
    constructor(containerComponentRef, 
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    moduleRef, data) {
        this.containerComponentRef = containerComponentRef;
        this.moduleRef = moduleRef;
        this.data = data;
        // eslint-disable-next-line @typescript-eslint/no-explicit-any
        this.close$ = new Subject();
    }
    /**
     * Triggers the dialog close process as well as informing @function @name onClose with the provided @param {output} via @member {Subject<any>} close$.
     *
     * @public
     * @param {?*} [output]
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    close(output) {
        this.close$.next(output);
        this.destroy();
    }
    /**
     * Informs the consumer with the provided data about the dialog close event.
     *
     * @public
     * @async
     * @returns {Promise<any>}
     */
    // eslint-disable-next-line @typescript-eslint/no-explicit-any
    async onClose() {
        // eslint-disable-next-line @typescript-eslint/no-unsafe-return
        return await firstValueFrom(this.close$);
    }
    /**
     * Destroys the dialog container, module reference and close$.
     *
     * @private
     */
    destroy() {
        this.containerComponentRef.destroy();
        this.moduleRef?.destroy();
        this.close$.complete();
    }
}

/**
 * Defines the structure of a NgModule's class required by @type {LazyDialogService}.
 *
 * @export
 * @abstract
 * @class ModuleWithLazyDialog
 * @typedef {ModuleWithLazyDialog}
 * @template T
 */
class ModuleWithLazyDialog {
}

/**
 * The LazyDialogService responsible for creating lazy loading dialogs.
 *
 * @export
 * @class LazyDialogService
 * @typedef {LazyDialogService}
 */
class LazyDialogService {
    /**
     * Creates an instance of LazyDialogService.
     *
     * @constructor
     * @public
     * @param {Readonly<Document>} document
     * @param {RendererFactory2} rendererFactory
     * @param {Injector} injector
     * @param {ApplicationRef} applicationRef
     */
    constructor(document, rendererFactory, injector, applicationRef) {
        this.document = document;
        this.rendererFactory = rendererFactory;
        this.injector = injector;
        this.applicationRef = applicationRef;
        this.renderer = this.rendererFactory.createRenderer(null, null);
    }
    /**
     * Creates (opens) a lazy loading dialog with provided data.
     *
     * @public
     * @async
     * @template ComponentType
     * @template DataType
     * @param {(Promise<Type<ModuleWithLazyDialog<ComponentType> | ComponentType>>)} module The module containing the lazy dialog component (supports NgModule extending ModuleWithLazyDialog<T> as well as standalone components).
     * @param {?DataType} [data] The data provided to the dialog.
     * @returns {Promise<LazyDialogRef<DataType>>}
     */
    async create(module, data) {
        const { componentType, moduleRef } = await this.initModuleAndComponent(module);
        const dialogContainerRootElement = this.createDialogRootElement();
        const { lazyDialogRef, dialogContainerVcr } = await this.initDialogBackgroundOverlay(dialogContainerRootElement, moduleRef, data);
        this.initDialog(lazyDialogRef, dialogContainerVcr, componentType, moduleRef);
        return lazyDialogRef;
    }
    /**
     * Determines the dialog component type (if @param module is a NgModule it also initializes the module).
     *
     * @private
     * @async
     * @template ComponentType
     * @param {(Promise<Type<ModuleWithLazyDialog<ComponentType> | ComponentType>>)} module The module containing the lazy dialog component (supports NgModule extending ModuleWithLazyDialog<T> as well as standalone components).
     * @returns {(Promise<{
        componentType: Type<ComponentType>;
        moduleRef: NgModuleRef<ModuleWithLazyDialog<ComponentType>> | undefined;
      }>)}
     */
    async initModuleAndComponent(module) {
        const resolvedModule = await module;
        let componentType;
        let moduleRef;
        if (resolvedModule.prototype instanceof (ModuleWithLazyDialog)) {
            const castedModule = resolvedModule;
            moduleRef = createNgModule(castedModule, this.injector);
            componentType = moduleRef.instance?.getDialog();
        }
        else {
            componentType = resolvedModule;
        }
        return {
            componentType,
            moduleRef,
        };
    }
    /**
     * Attaches the dialog to the DOM.
     *
     * @private
     * @template ComponentType
     * @template DataType
     * @param {LazyDialogRef<DataType>} lazyDialogRef The reference of the lazy dialog.
     * @param {ViewContainerRef} dialogContainerVcr The dialog container's view container reference.
     * @param {Type<ComponentType>} componentType The type of the dialog component.
     * @param {(NgModuleRef<ModuleWithLazyDialog<ComponentType>> | undefined)} moduleRef The reference of the module containing the dialog component (only has a value if module isn't a standalone component).
     */
    initDialog(lazyDialogRef, dialogContainerVcr, componentType, moduleRef) {
        const injector = Injector.create({
            parent: this.injector,
            providers: [{ provide: LazyDialogRef, useValue: lazyDialogRef }],
        });
        dialogContainerVcr.createComponent(componentType, {
            injector,
            ngModuleRef: moduleRef,
        });
    }
    /**
     * Attaches the dialog container to the DOM.
     *
     * @private
     * @async
     * @template ComponentType
     * @template DataType
     * @param {HTMLDivElement} dialogContainerRootElement The HTML element of the dialog container root.
     * @param {(NgModuleRef<ModuleWithLazyDialog<ComponentType>> | undefined)} moduleRef The reference of the module containing the dialog component (only has a value if module isn't a standalone component).
     * @param {?DataType} [data] The data provided to the dialog.
     * @returns {Promise<{
        lazyDialogRef: LazyDialogRef<DataType>;
        dialogContainerVcr: ViewContainerRef;
      }>}
     */
    async initDialogBackgroundOverlay(dialogContainerRootElement, moduleRef, data) {
        const dialogContainerComponent = await Promise.resolve().then(function () { return lazyDialogContainer_component; }).then((m) => m.LazyDialogContainerComponent);
        const dialogContainerComponentRef = this.applicationRef.bootstrap(dialogContainerComponent, dialogContainerRootElement);
        const lazyDialogRef = new LazyDialogRef(dialogContainerComponentRef, moduleRef, data);
        const dialogContainerVcr = dialogContainerComponentRef.instance.dialogContainer;
        return { lazyDialogRef, dialogContainerVcr };
    }
    /**
     * Creates the HTML dialog root container element.
     *
     * @private
     * @returns {HTMLDivElement}
     */
    createDialogRootElement() {
        const dialogContainer = this.renderer.createElement('div');
        this.renderer.addClass(dialogContainer, 'dialog-root');
        this.renderer.appendChild(this.document.body, dialogContainer);
        return dialogContainer;
    }
}
LazyDialogService.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.1.2", ngImport: i0, type: LazyDialogService, deps: [{ token: DOCUMENT }, { token: i0.RendererFactory2 }, { token: i0.Injector }, { token: i0.ApplicationRef }], target: i0.ɵɵFactoryTarget.Injectable });
LazyDialogService.ɵprov = i0.ɵɵngDeclareInjectable({ minVersion: "12.0.0", version: "14.1.2", ngImport: i0, type: LazyDialogService });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.1.2", ngImport: i0, type: LazyDialogService, decorators: [{
            type: Injectable
        }], ctorParameters: function () { return [{ type: undefined, decorators: [{
                    type: Inject,
                    args: [DOCUMENT]
                }] }, { type: i0.RendererFactory2 }, { type: i0.Injector }, { type: i0.ApplicationRef }]; } });

/**
 * The LazyDialogModule
 *
 * @export
 * @class LazyDialogModule
 * @typedef {LazyDialogModule}
 */
class LazyDialogModule {
    /**
     * Creates an instance of LazyDialogModule.
     *
     * @constructor
     * @public
     * @param {LazyDialogModule} parentModule
     */
    constructor(parentModule) {
        if (parentModule) {
            throw new Error('LazyDialogModule is already loaded. Import it in the AppModule only');
        }
    }
    /**
     * Creates the singleton services.
     *
     * @public
     * @static
     * @param {{
        [klass: string]: any;
     * }} dialogContainerStyles
     * @returns {ModuleWithProviders<LazyDialogModule>}
     */
    static forRoot(dialogContainerStyles) {
        return {
            ngModule: LazyDialogModule,
            providers: [
                LazyDialogService,
                {
                    provide: LAZY_DIALOG_CONTAINER_STYLES,
                    useValue: dialogContainerStyles,
                },
            ],
        };
    }
}
LazyDialogModule.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "14.1.2", ngImport: i0, type: LazyDialogModule, deps: [{ token: LazyDialogModule, optional: true, skipSelf: true }], target: i0.ɵɵFactoryTarget.NgModule });
LazyDialogModule.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "14.1.2", ngImport: i0, type: LazyDialogModule, declarations: [LazyDialogContainerComponent], imports: [CommonModule] });
LazyDialogModule.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "14.1.2", ngImport: i0, type: LazyDialogModule, imports: [CommonModule] });
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "14.1.2", ngImport: i0, type: LazyDialogModule, decorators: [{
            type: NgModule,
            args: [{
                    declarations: [LazyDialogContainerComponent],
                    imports: [CommonModule],
                }]
        }], ctorParameters: function () { return [{ type: LazyDialogModule, decorators: [{
                    type: Optional
                }, {
                    type: SkipSelf
                }] }]; } });

/**
 * Generated bundle index. Do not edit.
 */

export { LAZY_DIALOG_CONTAINER_STYLES, LazyDialogModule, LazyDialogRef, LazyDialogService, ModuleWithLazyDialog };
//# sourceMappingURL=ngx-toolset-lazy-dialogs.mjs.map
